"""Re-exports from typed_dicts for backward compatibility.

.. deprecated::
    All source-code imports from this module have been migrated to
    ``trw_mcp.models.typed_dicts`` (the canonical location).  This shim is
    retained **only** to avoid breaking external consumers.  Do NOT add new
    imports here — import directly from ``trw_mcp.models.typed_dicts`` instead.

All TypedDicts originally defined here have been consolidated into
``trw_mcp.models.typed_dicts``.  This module re-exports them so existing
``from trw_mcp.models.types import ...`` call-sites continue to work without
modification.
"""

from trw_mcp.models.typed_dicts import (
    AggregateMetrics,
    AnalyticsReport,
    ApiFuzzResult,
    AuditCeremonyComplianceResult,
    AuditDuplicatePairDict,
    AuditDuplicatesResult,
    AuditFixActionsDict,
    AuditHookVersionsResult,
    AuditIndexConsistencyResult,
    AuditLearningsResult,
    AuditRecallEffectivenessResult,
    AuditReflectionComponentsDict,
    AuditReflectionDiagnosticsDict,
    AuditReflectionQualityResult,
    AuditReport,
    AuditTelemetryBloatDict,
    AutoProgressStepResult,
    AutoRecalledItemDict,
    AutoReviewResult,
    BatchDedupResult,
    BatchSendResult,
    CeremonyApproveResult,
    CeremonyClassStatusDict,
    CeremonyFeedbackEntry,
    CeremonyFeedbackStepResult,
    CeremonyRevertResult,
    CeremonyScoreResult,
    CeremonyStatusResult,
    CeremonyTrendItem,
    CeremonyTrendResult,
    CheckpointEventDataDict,
    CheckpointRecordDict,
    ConsolidationStepResult,
    CoverageTrendResult,
    DedupHandleResult,
    DegradationAlertResult,
    DepAuditResult,
    DeployFrameworksVersionDataDict,
    EmbedHealthStatus,
    EscalationResult,
    ExportAnalyticsSection,
    ExportMetadata,
    ExportPatternsSection,
    ExportRunsSection,
    ExportSummary,
    FinalizeRunResult,
    HumanReviewResult,
    ImpactDistributionResult,
    ImpactTierInfo,
    IndexSyncResult,
    LearnResult,
    MultiReviewerAnalysisResult,
    NpmAuditResult,
    OutcomeCorrelationStepResult,
    PipAuditResult,
    ProgressionItem,
    PublishLearningsResult,
    PublishResult,
    RecallOutcomeStepResult,
    ReconcileReviewResult,
    RemoteSharedLearningDict,
    ReviewModeResult,
    ReviewTrendResult,
    RoadmapSyncResult,
    RunAnalysisResult,
    SessionRecallExtrasDict,
    StatusReflectionDict,
    StatusReversionLatestDict,
    StatusReversionMetricsDict,
    SyncIndexMdResult,
    TelemetryStepResult,
    TierCeremonyScoreResult,
    TierDistribution,
    TierMetrics,
    TierSweepStepResult,
    TrustIncrementResult,
    TrustLevelResult,
    TrustSessionIncrementResult,
    TrwInitConfigDataDict,
    TrwStatusDict,
    WaveDetailDict,
    WaveProgressDict,
    WaveShardCountsDict,
)

__all__ = [
    "AggregateMetrics",
    "AnalyticsReport",
    "ApiFuzzResult",
    "AuditCeremonyComplianceResult",
    "AuditDuplicatePairDict",
    "AuditDuplicatesResult",
    "AuditFixActionsDict",
    "AuditHookVersionsResult",
    "AuditIndexConsistencyResult",
    "AuditLearningsResult",
    "AuditRecallEffectivenessResult",
    "AuditReflectionComponentsDict",
    "AuditReflectionDiagnosticsDict",
    "AuditReflectionQualityResult",
    "AuditReport",
    "AuditTelemetryBloatDict",
    "AutoProgressStepResult",
    "AutoRecalledItemDict",
    "AutoReviewResult",
    "BatchDedupResult",
    "BatchSendResult",
    "CeremonyApproveResult",
    "CeremonyClassStatusDict",
    "CeremonyFeedbackEntry",
    "CeremonyFeedbackStepResult",
    "CeremonyRevertResult",
    "CeremonyScoreResult",
    "CeremonyStatusResult",
    "CeremonyTrendItem",
    "CeremonyTrendResult",
    "CheckpointEventDataDict",
    "CheckpointRecordDict",
    "ConsolidationStepResult",
    "CoverageTrendResult",
    "DedupHandleResult",
    "DegradationAlertResult",
    "DepAuditResult",
    "DeployFrameworksVersionDataDict",
    "EmbedHealthStatus",
    "EscalationResult",
    "ExportAnalyticsSection",
    "ExportMetadata",
    "ExportPatternsSection",
    "ExportRunsSection",
    "ExportSummary",
    "FinalizeRunResult",
    "HumanReviewResult",
    "ImpactDistributionResult",
    "ImpactTierInfo",
    "IndexSyncResult",
    "LearnResult",
    "MultiReviewerAnalysisResult",
    "NpmAuditResult",
    "OutcomeCorrelationStepResult",
    "PipAuditResult",
    "ProgressionItem",
    "PublishLearningsResult",
    "PublishResult",
    "RecallOutcomeStepResult",
    "ReconcileReviewResult",
    "RemoteSharedLearningDict",
    "ReviewModeResult",
    "ReviewTrendResult",
    "RoadmapSyncResult",
    "RunAnalysisResult",
    "SessionRecallExtrasDict",
    "StatusReflectionDict",
    "StatusReversionLatestDict",
    "StatusReversionMetricsDict",
    "SyncIndexMdResult",
    "TelemetryStepResult",
    "TierCeremonyScoreResult",
    "TierDistribution",
    "TierMetrics",
    "TierSweepStepResult",
    "TrustIncrementResult",
    "TrustLevelResult",
    "TrustSessionIncrementResult",
    "TrwInitConfigDataDict",
    "TrwStatusDict",
    "WaveDetailDict",
    "WaveProgressDict",
    "WaveShardCountsDict",
]
